// --- [ ir_sla_transmit.c ] 18T ------------------------------------------------
//       tab = 3, 24.01.09
// 
// fr den Slave genderte Routine zum Senden von 1 (!) Byte via Manchester-Codierung auf 36 KHz
// nicht kompatibel mit RC5-Standard !!

// Startsequenz: ein Impuls mit 1/2 Taktlnge,
// an der folgenden steigenden Flanke wird die Zeit synchroniert,
// es folgt dann noch eine halbe Bitzeit low, dann folgen die Daten 

// Das linke Bit wird getestet:
// - ist es eine 1, dann wird eine low/high Sequenz gesendet,
// - ist es eine 0, dann wird eine high/low Sequenz gesendet,
// danach wird das Word nach links geschoben.
// die Dauer eines Bits betrgt 0.5ms, das sind 18 Takte bei 36KHz low sowie
// 18 Takte high: (18 + 18) / 36000 = 36 / 36000 = 1000us 

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "ir_slave.h"


// wenn die LED nach Plus (highside) geschaltet wird, aktivieren, bei lowside auskommentieren
// sorgt dafr, dass nach dem Start die LED in der ersten Impulshlfe eingeschaltet ist ...
// ist vermutlich berflssig ...
//#define HIGHSIDE

// ----------------------------------------------------------------------------
// Warten einen halben Takt
// _delay_us kann nur bis 768us/MHz verzgern, somit sind bei 8 MHz max. 96us erreichbar
// Dauer = 18/36000 = 500us
void wait_hT(void)
{
register uint8_t i;
for(i = 0; i < 10; i++)
	{
	_delay_us(50);								// 500us
	}
}

// ----------------------------------------------------------------------------
// sendet 1 Startbit und data (8Bit)
void IR_sla_transmit(uint8_t data)
{
register uint8_t count;
													// Empfang stoppen
TIMSK  = 0;										// Timer0 Interrupt abschalten
TCCR0B = 0;										// Timer0 stoppen
GIMSK  = 0;										// INT0 Interrupt abschalten

cli();											// Interrupts stren beim Senden nur

TCCR0A = IR_CONFIG;							// Register fr die Frequenzerzeugung setzen
OCR0A  = COMPARE;								// Comparewert fr toggle OC0A und TCNT0 reset

// Startbit: 1/2 Takt auf low (Senden), danach 1/2 Takt auf high(Pause)
													// 1. Taktteil LED aktiv
		TCNT0 = 0;								// Zhler zurcksetzen
#ifdef HIGHSIDE
		IR_PORT &= (~(1<<IR_PIN));			// LED einschalten (auf low setzen)
#else
		IR_PORT |= (1<<IR_PIN);				// LED einschalten (auf high setzen)
#endif
		DDR_IR_PORT |= (1<<IR_PIN);		// Pin auf Ausgang
													
		TCCR0B = PWM_ON;						// PWM starten
		wait_hT();								// halben Takt senden 

													// 2. Taktteil Pause
		TCCR0B = PWM_OFF;						// PWM stoppen -> der Empfnger beginnt hier mit der Zhlung
		DDR_IR_PORT &= (~(1<<IR_PIN));	// DDR auf Eingang
#ifdef HIGHSIDE
		IR_PORT |= (1<<IR_PIN);				// LED ausschalten (auf high setzen)
#else
		IR_PORT &= (~(1<<IR_PIN));			// LED ausschalten (auf low setzen)
#endif		
		wait_hT();		
													// Jetzt folgen die Daten

for(count = 0; count < IR_BITS; count++)// Anzahl der zu sendenden Bits regelt count
	{
	if (data & (1<<(IR_BITS - 1)))		// wenn das 'linkste' Bit gesetzt ist, dann
		{											// eine '1' ausgeben, erst low, dann high
		
													// 1. Takthlfte low
		TCCR0B = PWM_OFF;						// PWM stoppen
		DDR_IR_PORT &= (~(1<<IR_PIN));	// DDR auf Eingang

#ifdef HIGHSIDE
		IR_PORT |= (1<<IR_PIN);				// LED ausschalten (auf high setzen)
#else
		IR_PORT &= (~(1<<IR_PIN));			// LED ausschalten (auf low setzen)
#endif
		wait_hT();
													// 2. Takthlfte high
		TCNT0 = 0;								// Zhler zurcksetzen
#ifdef HIGHSIDE
		IR_PORT &= (~(1<<IR_PIN));			// LED einschalten (auf low setzen)
#else
		IR_PORT |= (1<<IR_PIN);				// LED einschalten (auf high setzen)
#endif
		DDR_IR_PORT |= (1<<IR_PIN);		// Pin auf Ausgang
		TCCR0B = PWM_ON;						// PWM starten
		wait_hT();								// der Timer luft beim Verlassen noch !!	
		}
	else											// wenn low-Byte von data  = 0, dann
		{											// eine '0' senden, erst high, dann low

		TCNT0 = 0;								// 1. Takthlfte high
#ifdef HIGHSIDE
		IR_PORT &= (~(1<<IR_PIN));			// LED einschalten (auf low setzen)
#else
		IR_PORT |= (1<<IR_PIN);				// LED einschalten (auf high setzen)
#endif		
		DDR_IR_PORT |= (1<<IR_PIN);		// Portpin auf Ausgang
		TCCR0B = PWM_ON;						// PWM starten
		wait_hT();
													// 2. Takthlfte low
		TCCR0B = PWM_OFF;
		DDR_IR_PORT &= (~(1<<IR_PIN));	// Portpin als Eingang schalten

#ifdef HIGHSIDE
		IR_PORT |= (1<<IR_PIN);				// LED ausschalten (auf high setzen)
#else
		IR_PORT &= (~(1<<IR_PIN));			// LED ausschalten (auf low setzen)
#endif
		wait_hT();
		}
	
	data = data << 1;							// Bits nach links schieben
	}												// for i < anzahl_bits	
	
	TCCR0B = PWM_OFF;							// PWM  abschalten
	DDR_IR_PORT &= (~(1<<IR_PIN));		// den IR_Pin als Eingang schalten
	IR_PORT &= (~(1<<IR_PIN));				// Portpin auf low, Pullup aus
}

// --- [ end of file ] --------------------------------------------------------
